/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.allgemein;
import java.io.NotActiveException;
import java.io.Serializable;

import javax.activation.UnsupportedDataTypeException;

/**
 * (Familien-)Interface fr konfigurierbare Daten. Ermglicht es, einfach auf Konfigurationsdaten
 * mit unterschiedlichen technischen Speichermethoden (bspw. per Datei, per Datenverteiler-Objekte,
 * per Datenverteiler-Parametrierung) zuzugreifen. 
 * Die Daten sollen grundstzlich in einem Schlssel-Wert-Paar vorliegen. Fr jedes Plugin ist
 * ein eigener "Konfigurationsbereich" (kein Zusammenhang mit Begriff des Datenverteilers)
 * vorgesehen, der hier "Kontext" heit.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public interface KonfigurationsModul {
  /**
   * Liefert die Bezeichnung des Konfigurationskontexts zurck.
   * 
   * @return Kontext
   */
  String getContext();
  
  /**
   * Prft, ob zu einem gegebenen Schlssel ein Konfigurationsdatum existiert.
   * 
   * @param key Schlssel
   * @return true: es existiert ein Konfigurationsdautm
   */
  boolean hasData(String key);
  
  /**
   * Allgemeine Methode zum Abruf von Konfigurationsdaten. Die Daten werden allgemein
   * als String abgelegt. Wird eine andere Klasse als classx angegeben, wird eine
   * Konvertierung durchgefhrt (falls eine solche nicht mglich oder nicht implementiert
   * ist, wird eine UnsupportedDataTypeException geworfen).
   * Diese Methode wirft bei nicht-vorhandenem Schlssel eine Exception (statt Rckgabe von null).
   * 
   * @param key Schlssel
   * @param classx Typ des Werts
   * @param <T> Typ des Werts
   * @return Wert zum Schlssel
   * @throws UnsupportedDataTypeException
   *            Konvertierung in geforderten Datentyp nicht implementiert
   * @throws IllegalArgumentException
   *            Konvertierung in geforderten Datentyp nicht mglich (falsche Konfigurationsdaten)
   * @throws NotActiveException
   *            Auf die Konfigurationsdaten kann allgemein nicht zugegriffen werden.
   */
  
  <T extends Serializable> T getData(String key, Class<T> classx) 
      throws UnsupportedDataTypeException, NotActiveException, IllegalArgumentException;  

  /**
   * Rckgabe eines Konfigurationsdatum als String. Ist zum Schlssel kein Wert vorhanden,
   * wird null zurckgegeben.
   * 
   * @param key Schlssel
   * @return Wert zum Schlssel oder null (wenn nicht vorhanden)
   */
  
  String getString(String key); 
  
  /**
   * Rckgabe eines Konfigurationsdatum als Integer. Ist zum Schlssel kein Wert vorhanden,
   * wird null zurckgegeben.
   * 
   * @param key Schlssel
   * @return Wert zum Schlssel oder null (wenn nicht vorhanden)
   */
  
  Integer getInteger(String key);
}
