/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.datensammlung;
import java.util.Collection;
import java.util.Map;

import javax.naming.ConfigurationException;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;

/**
 * Schnittstellendefinition fr Methode zur Sammlung von dynamischen Daten ber den Datenverteiler.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */
public interface DatenSammlungsMethode {
  /**
   * Liefert true zurck, wenn die vorliegenden Daten publiziert werden knnen.
   * 
   * @return true/false wie angegeben.
   */
  boolean isSendeBereit();
  
  /**
   * Rckgabe einer Liste mit aktuellen Daten (fr jede Kombination Objektliste/Attributgruppe/Aspekt getrennt.
   * 
   * @return Map wie skizziert.
   */
  Map<ObjekteAtgAsp, Collection<ResultData> > ermittleDaten();
  
  /**
   * Initialisiere die Instanz. Sollte insbesondere die Subskription beim Datenverteiler fr die
   * bergebenen Objektlisten umfassen.
   * 
   * @param objekteAtgAspListe Liste von Objektliste/Attributgruppe/Aspekt-Kombinationen
   * @return true, wenn erfolgreich initialisiert.
   * @throws ConfigurationException Bei Fehlern in Konfigurationsdaten.
   */
  boolean initialisiere(Collection<ObjekteAtgAsp> objekteAtgAspListe) throws ConfigurationException;
  
  /**
   * Methode sollte nach erfolgreicher Publizierung aufgerufen werden (typischer Anwendungsfall:
   * Bereinigung eines lokalen Caches).
   */
  void wurdePubliziert();
}