/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.definition;

import javax.naming.ConfigurationException;

import de.bsvrz.kex.datex2.plugins.empfang.EmpfangsMethode;
import de.bsvrz.kex.datex2.plugins.konvertierung.DaVKonvertierungDynamisch;


/**
 * Abstrakte Klasse fr Plugin-Definitionen (fr den Import von Daten).
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public abstract class PluginImport extends KExD2Plugin {
  /** Referenz auf Konvertierungsklassen-Instanz. */
  protected DaVKonvertierungDynamisch davKonvertierungDynamisch;
  
  /** Referenz auf Empfangsmethodenklassen-Instanz. */
  protected EmpfangsMethode empfangsMethode;

  /**
   * Konstruktor.
   * 
   * @param standardPeriod Zeitspanne zwischen zwei Import-Aufrufen.
   */
  
  protected PluginImport(final int standardPeriod) {
    super(standardPeriod);
    instantiiereDavKonvertierungDynamisch();
    if (this.davKonvertierungDynamisch == null) {
      throw new IllegalStateException("Das Plugin hat keine Methode fr die Konvertierung dynamischer Daten angegeben.");
    }
  
    instantiierePluginAusfuehrungsAlgorithmus();
    if (this.pluginAusfuehrungsAlgorithmus == null) {
      throw new IllegalStateException("Das Plugin hat keine Methode fr die Ausfhrung des Plugins angegeben.");
    }

    instantiiereEmpfangsMethode();  
  }

  /** 
   * Initialisiere alle assoziierten Objekte.
   * 
   * @throws ConfigurationException Fehler bei der Initialisierung
   */
  
  public void initialisiereAbhaengigkeiten() throws ConfigurationException {
    empfangsMethode.initialisiere();
    davKonvertierungDynamisch.initialisiere();
  }

  /**
   * Abstrakte Methode / Erzwingt Instantiierung des Plugin-Empfangsmethode.
   */
  protected abstract void instantiiereEmpfangsMethode();
  
  /**
   * Abstrakte Methode / Erzwingt Instantiierung des Plugin-Ausfhrungsalgorithms.
   */
  protected abstract void instantiierePluginAusfuehrungsAlgorithmus();
  
  /**
   * Abstrakte Methode / Erzwingt Instantiierung einer Konvertierungsklassen-Instanz.
   */
  protected abstract void instantiiereDavKonvertierungDynamisch();

  /**
   * Rckgabe der dynamischen Konvertierungsklassen-Instanz.
   * 
   * @return Konvertierungsklasse.
   */
  public DaVKonvertierungDynamisch getDaVKonvertierungDynamisch() {
    return davKonvertierungDynamisch;
  }

  /**
   * Rckgabe der dynamischen Empfangsmethodenklassen-Instanz.
   * 
   * @return Konvertierungsklasse.
   */
  
  public EmpfangsMethode getEmpfangsMethode() {
    return empfangsMethode;
  }

  /**
   * Initialisierung des Plugins.
   * 
   * @throws ConfigurationException Plugin konnte wegen Konfigurationsfehlern nicht initialisiert werden.
   */
    
  public abstract void initialisiere() throws ConfigurationException;
  
  /**
   * Rckgabe, ob Plugin erfolgreich initialisiert wurde.
   * 
   * @return true, wenn Plugin erfolgreich initialisiert wurde.
   */
    
  public final boolean isInitialisiert() {
	  return getPluginAusfuehrungsAlgorithmus() != null;
  }
}
