/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung;
import java.util.Collection;
import java.util.Map;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;

/**
 * Schnittstellendefinition fr Konvertierungsklassen zur bersetzung von dynamischen Daten
 * vom Datenverteiler-Format in das DATEX-II Format.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */
public interface DatexKonvertierungDynamisch {
  /**
   * Fhre Konvertierung von Datenverteiler-Daten nach DATEX-II durch.
   * 
   * @param data Objekte (bzw. Objekttyp, Attributgruppe und Aspekt) in Relation mit empfangenen Daten (ResultData)
   * @return JAX-B Struktur (Kontext ber getJAXBContext() abrufbar)
   */

  Object getDatexII(Map<ObjekteAtgAsp, Collection<ResultData>> data);

  /**
   * Rckgabe des JAX-B-Kontexts fr die in liefereDaten verwendete JAX-B-Struktur.
   * 
   * @return JAXBContext
   * @throws JAXBException Falls die Instantiierung des Kontext nicht mglich ist.
   */
  JAXBContext getJAXBContext() throws JAXBException;
  
  /**
   * Initialisiere die Instanz mit einer Liste von Objektliste/Attributgruppe/Aspekt-Kombination.
   * Die Initialisierung kann die bergebene Liste objects auch ndern, wenn darin enthaltene
   * Objekte den festgelegten Kriterien nicht gengen.
   * 
   * @param objects Liste von Objektliste/Attributgruppe/Aspekt-Kombination.
   * @throws ConfigurationException Das Plugin kann nicht initialisiert werden.
   */

  void initialisiere(Collection<ObjekteAtgAsp> objects) throws ConfigurationException;
}
