/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.ConfidentialityValueEnum;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.CountryEnum;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.D2LogicalModel;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.Exchange;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.HeaderInformation;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.InformationStatusEnum;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.InternationalIdentifier;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.PayloadPublication;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.Point;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.PointByCoordinates;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.PointCoordinates;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.PredefinedLocation;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.PredefinedLocationsPublication;
import eu.datex2.schema.statisch.predefined.locations.messquerschnitte.PredefinedNonOrderedLocationGroup;

/**
 * Konvertierung von statischen Daten (Messquerschnittsverortung) in das DATEX-II Format.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class MQPredefinedLocation extends StandardKonvertierungStatisch {

  /**
   * Konstruktor.
   * 
   * @param plugin Referenz auf Plugin-Definitions-Klasse
   */
  public MQPredefinedLocation(final PluginExport plugin) {
    this.plugin = plugin;
  }

  /***************************************************************************
   * IN DEM FOLGENDEN ABSCHNITT WERDEN STRUKTURELL GLEICHE METHODEN GESAMMELT.
   * 
   * Diese Methoden werden in den einzelnen Plugins wiederholt - allerdings
   * beziehen sich die JAXB-Elemente trotz gleichen Namens dann auf
   * andere Klassen; eine nominale Gleichheit existiert also nicht.
   ***************************************************************************
   */

  /**
   * Gebe den in dieser Klasse verwendeten JAXBContext zurck. 
   * 
   * @throws JAXBException {@see javax.xml.bind.JAXBContext#newInstance(String)}
   * @return JAXBContext Verwendeter Kontext
   */

  @Override
  public JAXBContext getJAXBContext() throws JAXBException {
    if (jaxbContext == null) {
      jaxbContext = JAXBContext.newInstance(D2LogicalModel.class.getPackage().getName());
    }
    return jaxbContext;
  }

  /**
   * Setzt die allgemeinen Angaben zur Publikation (Exchange, Language, PublicationCreator).
   * 
   * @param d2l D2LogicalModel
   * @param payload Verwendeter Payload
   */

  private void setzeNationalIdentifier(final D2LogicalModel d2l, final PayloadPublication payload) {
    InternationalIdentifier internationalIdentifier = new InternationalIdentifier();
    internationalIdentifier.setCountry(CountryEnum.DE);
    internationalIdentifier.setNationalIdentifier(nationalIdentifier);
    
    // Setze Exchange
    
    Exchange exchange = new Exchange();
    exchange.setSupplierIdentification(internationalIdentifier);
    d2l.setExchange(exchange);

    // Setze Language
    payload.setLang("DE");

    payload.setPublicationCreator(internationalIdentifier);
  }

  /**
   * Setzt Koordinaten aus Data in ein PointCoordinates Objekt.
   * 
   * @param data
   *          Data
   * @return PointCoordinates
   */
  
  private PointCoordinates setzeKoordinaten(final Data data) {
    PointCoordinates pointCoordinates = new PointCoordinates();
    pointCoordinates.setLatitude(WerkzeugSammlung.getLatitude(data));
    pointCoordinates.setLongitude(WerkzeugSammlung.getLongitude(data));
    return pointCoordinates;
  }
  
  /***********************************************************************
   * ENDE DES ABSCHNITTS STRUKTURELL GLEICHER METHODEN
   ***********************************************************************
   */

  /** {@inheritDoc} */
  @Override
  public Object getDatexII(final Collection<ObjekteAtgAsp> objekteAtgAspListe, final CreationOptions... options) {
    boolean versionFree = WerkzeugSammlung.isVersionFree(options);
    String newVersion = WerkzeugSammlung.erzeugeVersionsString(versionFree);
    
    D2LogicalModel d2l = new D2LogicalModel();
    PredefinedLocationsPublication payload = new PredefinedLocationsPublication();
    d2l.setPayloadPublication(payload);
    
    setzeNationalIdentifier(d2l, payload);
    
    setzeHeaderInformation(versionFree, payload);

  	PredefinedNonOrderedLocationGroup predefinedNonOrderedLocationGroup = new PredefinedNonOrderedLocationGroup();
    predefinedNonOrderedLocationGroup.setId(locationGroupId);
    predefinedNonOrderedLocationGroup.setVersion(newVersion);
    payload.getPredefinedLocationContainers().add(predefinedNonOrderedLocationGroup);

    for (ObjekteAtgAsp objekteAtgAsp : objekteAtgAspListe) {
      for (SystemObject mq : objekteAtgAsp.getSystemObjekte()) {
        AttributeGroup attributGruppe = mq.getDataModel().getAttributeGroup("atg.punktKoordinaten");
        Data d = mq.getConfigurationData(attributGruppe);
        PointCoordinates pointCoordinates = setzeKoordinaten(d);

        PredefinedLocation predefinedLocation = new PredefinedLocation();
        predefinedLocation.setVersion(newVersion);
        String pid = mq.getPid();
        predefinedLocation.setId(pid);
        Point point = new Point(); // extends NetworkLocation -> extends Location
        PointByCoordinates pointByCoordinates = new PointByCoordinates();
        pointByCoordinates.setPointCoordinates(pointCoordinates);
        point.setPointByCoordinates(pointByCoordinates);
        predefinedLocation.setLocation(point);
        predefinedNonOrderedLocationGroup.getPredefinedLocations().add(predefinedLocation);     
      }
    }
        
    if (istGueltig(predefinedNonOrderedLocationGroup)) {
      persistiereVersion(versionFree, newVersion);
      return d2l;
    } else {
      return null;
    }
  }
  
  /**
   * Setze die HeaderInformation des bergebenen Payloads.
   * 
   * @param versionFree true, wenn keine Versionsinformationen eingetragen werden sollen.
   * @param payload Zu ergnzender Payload
   */

  private void setzeHeaderInformation(final boolean versionFree, final PredefinedLocationsPublication payload) {
    HeaderInformation headerInformation = new HeaderInformation();
    headerInformation.setConfidentiality(ConfidentialityValueEnum.NO_RESTRICTION);
    headerInformation.setInformationStatus(InformationStatusEnum.REAL);
    payload.setHeaderInformation(headerInformation);    
    payload.setPublicationTime(WerkzeugSammlung.erzeugeVersionsdatum(versionFree));
  }
  
  /**
   * Prft, ob die Publikation grundstzlich gltig sein kann (hier: ist die Anzahl an Eintrgen dem Schema 
   * entsprechend ausreichend).
   * 
   * @param predefinedNonOrderedLocationGroup Zu prfende Struktur
   * @return true = die Struktur kann verwendet werden.
   */

  private boolean istGueltig(final PredefinedNonOrderedLocationGroup predefinedNonOrderedLocationGroup) {
    return predefinedNonOrderedLocationGroup.getPredefinedLocations().size() > 1;
  }

  @Override
  public void initialisiere(final Collection<ObjekteAtgAsp> objects) throws ConfigurationException {
    initialisiereBasisFelder();

    // Modifiziere Liste der Systemobjekte: Entferne alle Objekte, bei denen keine Koordinatenangabe mglich ist.

    for (ObjekteAtgAsp objekteAtgAsp : objects) {
      List <SystemObject> listeNeueSystemObjekte = new ArrayList<>();
      for (SystemObject mq : objekteAtgAsp.getSystemObjekte()) { 
        try {
          AttributeGroup attributGruppe = mq.getDataModel().getAttributeGroup("atg.punktKoordinaten");
          Data d = mq.getConfigurationData(attributGruppe);
          if (d != null) {
            setzeKoordinaten(d); // throws exception if anything goes wrong
            listeNeueSystemObjekte.add(mq);
          }
        } catch (Exception dummy) {
          LOGGER.finer(logPrefix + "Fehler in Koordinaten, entferne Eintrag " + mq.getPid());
        }
      } // for (SystemObject mq : objekteAtgAsp.getSystemObjekte())
      objekteAtgAsp.getSystemObjekte().clear();  
      objekteAtgAsp.getSystemObjekte().addAll(listeNeueSystemObjekte);
    } // for (ObjekteAtgAsp objekteAtgAsp : objekte)	
  }
}