/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.publizierung;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;

/**
 * Interface zur Ausgabe der DATEX-II-Struktur.
 *
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public interface PublizierungsMethode {
  /**
   * Liefert einen Namen der Publizierungsmethode zurck.
   * @return Publizierungsmethode (bspw. "MDM".
   */
	String getPublizierungsMethodenName();
	
  /**
   * Gebe DATEX-II Struktur aus.
   * 
   * @param xml JAX-B Struktur
   * @param context JAX-B Kontext
   * @param param Zustzlicher Parameter fr Methode 
   * @throws Exception Fehler bei der Publikation
   */
	void publiziere(Object xml, JAXBContext context, String param) throws Exception;
	

  /**
   * Initialisiere die implementierende Klasse. Diese Methode wird aufgerufen,
   * nachdem alle Abhngigkeiten in der assoziierten Plugin-Definitions-Klasse injiziert wurden.
   * 
   * @return true Initialisierung erfolgreich, false: Initialisierung fehlerhaft
   * @throws ConfigurationException Konfigurationsdaten sind fehlerhaft
   */  
  boolean initialisiere() throws ConfigurationException;

}
