package de.bsvrz.datex2.test.cases;
import java.util.Arrays;
import java.util.List;

import javax.naming.ConfigurationException;

import org.junit.Assert;
import org.junit.Test;

import de.bsvrz.datex2.test.factories.CreateMockMessquerschnitte;
import de.bsvrz.datex2.test.factories.CreateMockObjects;
import de.bsvrz.datex2.test.mockclasses.MockDavConnection;
import de.bsvrz.datex2.test.mockclasses.MockKonfigUndPersistenzModul;
import de.bsvrz.datex2.test.mockclasses.MockPublishMethod;
import de.bsvrz.kex.datex2.plugins.definition.impl.DaV_MDM_001;

public class WhiteBox_PlugIn_DaV_MDM_001 extends WhiteBox_PlugIns {
  
  public WhiteBox_PlugIn_DaV_MDM_001() {
    super("DATEXII_DaV-MDM-001_dyn.xsd",
          "DATEXII_DaV-MDM-001_stat.xsd");
  }

  @Override
  protected void populateConfigAndPersistence(MockKonfigUndPersistenzModul konfig, MockKonfigUndPersistenzModul persistenz) {
    konfig.putData("National-Identifier", "TestNI");
    konfig.putData("LocationGroupId", "TestGroupId");
    konfig.putData("StandardIntervall", "3.0");
    konfig.putData("Maximales-Alter", 3);
    persistenz.putData("HashCodeStaticXML", 123);
    persistenz.putData("PredefinedLocationVersion", "correct");
  }

  private List<Integer> listIntData = Arrays.asList(-1, -255, 255, Integer.MAX_VALUE, Integer.MIN_VALUE);

  /**
   * Standard Gutlauf
   * 
   * @throws Exception
   */
  @Test
  public void testGutlauf() throws Exception {
    CreateMockMessquerschnitte mqCreation = new CreateMockMessquerschnitte();
    MockDavConnection davConnection = new MockDavConnection(null, null);
    MockPublishMethod dynamicResult = new MockPublishMethod();
    MockPublishMethod staticResult = new MockPublishMethod();
    
    DaV_MDM_001 mockPluginExport = new DaV_MDM_001();

    init(mockPluginExport, davConnection, mqCreation, staticResult, dynamicResult);

    mockPluginExport.initialisiere();
    
    mockPluginExport.getPluginAusfuehrungsAlgorithmus().starte();

    Assert.assertTrue(validate(SCHEMA_URI_STAT, mockPluginExport.getDatexKonvertierungStatisch().getJAXBContext(), staticResult.result , CanBeNull.CANNOT_BE_NULL));

    mqCreation.mockSendDataToAll(davConnection, 34, CreateMockObjects.SendOptions.NORMAL);

    mockPluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();
    
    Assert.assertTrue(validate(SCHEMA_URI_DYN, mockPluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CANNOT_BE_NULL));
  }

  /**
   * Testlauf mit typischen fehlerhaften Werten (siehe Float-Liste oben).
   * 
   * @throws Exception
   */

  @Test
  public void testFehlerhafteDaten() throws Exception {
    CreateMockMessquerschnitte mqCreation = new CreateMockMessquerschnitte();
    MockDavConnection davConnection = new MockDavConnection(null, null);
    MockPublishMethod dynamicResult = new MockPublishMethod();
    MockPublishMethod staticResult = new MockPublishMethod();
    
    DaV_MDM_001 mockPluginExport = new DaV_MDM_001();

    init(mockPluginExport, davConnection, mqCreation, staticResult, dynamicResult);

    mockPluginExport.initialisiere();
    
    mockPluginExport.getPluginAusfuehrungsAlgorithmus().starte();

    Assert.assertTrue(validate(SCHEMA_URI_STAT, mockPluginExport.getDatexKonvertierungStatisch().getJAXBContext(), staticResult.result , CanBeNull.CANNOT_BE_NULL));

    for (Integer intData : listIntData) {
      mqCreation.mockSendDataToAll(davConnection, intData, CreateMockObjects.SendOptions.NORMAL);
      mockPluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();
      
      Assert.assertTrue(validate(SCHEMA_URI_DYN, mockPluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CAN_BE_NULL));
    }   
  }

  
  @Test
  public void testFehlerhafteDaten2() throws Exception {
    CreateMockMessquerschnitte mqCreation = new CreateMockMessquerschnitte();
    MockDavConnection davConnection = new MockDavConnection(null, null);
    MockPublishMethod dynamicResult = new MockPublishMethod();
    MockPublishMethod staticResult = new MockPublishMethod();
    
    DaV_MDM_001 mockPluginExport = new DaV_MDM_001();

    init(mockPluginExport, davConnection, mqCreation, staticResult, dynamicResult);

    mockPluginExport.initialisiere();
    
    mockPluginExport.getPluginAusfuehrungsAlgorithmus().starte();

    Assert.assertTrue(validate(SCHEMA_URI_STAT, mockPluginExport.getDatexKonvertierungStatisch().getJAXBContext(), staticResult.result , CanBeNull.CANNOT_BE_NULL));

    mqCreation.mockSendDataToAll(davConnection, 20, CreateMockObjects.SendOptions.HAS_NO_DATA);
    mockPluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();
    Assert.assertTrue(validate(SCHEMA_URI_DYN, mockPluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CAN_BE_NULL));

    mqCreation.mockSendDataToAll(davConnection, 20, CreateMockObjects.SendOptions.NULL_OBJ);
    mockPluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();
    Assert.assertTrue(validate(SCHEMA_URI_DYN, mockPluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CAN_BE_NULL));

  }
  
  /**
   * Fehlende obligatorische Parameter mssen zu einer Exception fhren.
   * 
   * @throws Exception
   */
  @Test(expected = ConfigurationException.class)
  public void testFehlendeParameterStatisch() throws Exception {
    testFehlendeParameterStatisch(new DaV_MDM_001(), "National-Identifier");
  }

  /**
   * Fehlende obligatorische Parameter mssen zu einer Exception fhren.
   * 
   * @throws Exception
   */
  @Test(expected = ConfigurationException.class)
  public void testFehlendeParameterStatisch2() throws Exception {
    testFehlendeParameterStatisch(new DaV_MDM_001(), "Maximales-Alter");
  }

  /**
   * Fehlende obligatorische Parameter mssen zu einer Exception fhren.
   * 
   * @throws Exception
   */
  @Test(expected = ConfigurationException.class)
  public void testFehlendeParameterStatisch3() throws Exception {
    testFehlendeParameterStatisch(new DaV_MDM_001(), "LocationGroupId");
  }
  
  /**
   * Wenn keine Umfelddatensensoren und -messstellen definiert sind, kann zwar kein XML erzeugt werden (= leeres Dokument),
   * die Exceptions mssen aber in den aufgerufenen Methoden abgefangen werden.
   * 
   * @throws Exception
   */
  @Test
  public void testKeineMessquerschnitte() throws Exception {
    super.testKeineObjekte(new DaV_MDM_001());
  }

}
