package de.bsvrz.datex2.test.cases.old;
import java.io.ByteArrayOutputStream;
import java.io.StringReader;
import java.lang.reflect.Field;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.naming.ConfigurationException;
import javax.xml.XMLConstants;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import de.bsvrz.datex2.test.mockclasses.MockData;
import de.bsvrz.datex2.test.mockclasses.MockDataNumberValue;
import de.bsvrz.datex2.test.mockclasses.MockDataTextValue;
import de.bsvrz.datex2.test.mockclasses.MockDataTimeValue;
import de.bsvrz.datex2.test.mockclasses.MockKonfigUndPersistenzModul;
import de.bsvrz.datex2.test.mockclasses.MockObjekteAtgAsp;
import de.bsvrz.datex2.test.mockclasses.MockPluginExport;
import de.bsvrz.datex2.test.mockclasses.MockSystemObject;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.MQPredefinedLocation;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.MQVerkehrslage;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.UDMeasurementSite;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.WerkzeugSammlung;

public class WBKonvertierung002d {

  private static final String TYP_MESSQUERSCHNITT = "typ.messQuerschnitt";
  private static final String ATG_VERKEHRSDATEN = "atg.strfallZustand";
  private static final String SCHEMA_URI = "file:///d:/home/workspace/KExDatexII/org/DATEXII_DaV-MDM-002_dyn.xsd";
  
  /** Datex II Namensraum */
  private final String DATEX2_NAMESPACE = "http://datex2.eu/schema/2/2_0";

  /** Root-Tag des Datex II Formats */
  private final String DATEX2_ROOTTAG = "d2LogicalModel";

  Collection<ResultData> ordinaryDataset = new ArrayList<ResultData>();
//  Collection<ResultData> boundaryDataset = new ArrayList<ResultData>();
  Collection<ResultData> faultDataset1 = new ArrayList<ResultData>();
  Collection<ResultData> faultDataset2 = new ArrayList<ResultData>();
  Collection<ResultData> faultDataset3 = new ArrayList<ResultData>();
//  Collection<ResultData> faultDataset4 = new ArrayList<ResultData>();
//  Collection<ResultData> faultDataset5 = new ArrayList<ResultData>();
  Collection<ResultData> faultDataset6 = new ArrayList<ResultData>();
  Collection<ResultData> emptyDataset = new ArrayList<ResultData>();

  MockObjekteAtgAsp ordinaryObjSet = new MockObjekteAtgAsp();
  MockObjekteAtgAsp faultObjSet1 = new MockObjekteAtgAsp();
  MockObjekteAtgAsp faultObjSet2 = new MockObjekteAtgAsp();
  MockObjekteAtgAsp emptyObjSet = new MockObjekteAtgAsp();
  private Schema schema;
  
  enum CanBeNull { CAN_BE_NULL, CANNOT_BE_NULL };

  @Before
  public void createDataSets() throws Exception {
    ordinaryDataset.add(createOrdinaryDataset1());
    for (String i : Arrays.asList("stockender Verkehr", "zhflieender Verkehr", "lebhafter Verkehr", "dichter Verkehr","Stau","keine Aussage","Strung","freier Verkehr","",null )) {
      ResultData d = createOrdinaryDataset1();
      d.getData().getTextValue("Situation").setText(i);      
      ordinaryDataset.add(d);
    }
    for (String i : Arrays.asList("stockender Verkehr", "zhflieender Verkehr", "lebhafter Verkehr", "dichter Verkehr","Stau","keine Aussage","Strung","freier Verkehr","",null )) {
      ResultData d = createOrdinaryDataset1();
      d.getData().getTextValue("Situation").setText(i);
      d.getData().getTimeValue("Horizont").setMillis(60000);
      ordinaryDataset.add(d);
    }
    for (String i : Arrays.asList("stockender Verkehr", "zhflieender Verkehr", "lebhafter Verkehr", "dichter Verkehr","Stau","keine Aussage","Strung","freier Verkehr","",null )) {
      ResultData d = createOrdinaryDataset1();
      d.getData().getTextValue("Situation").setText(i);
      d.getData().getItem("Gte").getUnscaledValue("Index").set(-1);
      ordinaryDataset.add(d);
    }
    faultDataset1.add(createFaultDataset1());
    faultDataset2.add(createFaultDataset2());
    faultDataset3.add(createFaultDataset3());
    faultDataset6.add(null);

    ordinaryObjSet = new MockObjekteAtgAsp();
    ordinaryObjSet.createMockTyp(TYP_MESSQUERSCHNITT, "Messquerschnitt");
    ordinaryObjSet.createAttributeGroup(ATG_VERKEHRSDATEN, "Verkehrsdaten");

    faultObjSet1 = new MockObjekteAtgAsp();
    faultObjSet1.createMockTyp("typ.nichtExistent", "Messquerschnitt");
    faultObjSet1.createAttributeGroup(ATG_VERKEHRSDATEN, "Verkehrsdaten");

    faultObjSet2 = new MockObjekteAtgAsp();
    faultObjSet2.createMockTyp(TYP_MESSQUERSCHNITT, "Messquerschnitt");
    faultObjSet2.createAttributeGroup("atg.nichtExistent", "Verkehrsdaten");
    
    emptyObjSet = new MockObjekteAtgAsp();

    URL schemaFile = new URL(SCHEMA_URI);
    SchemaFactory schemaFactory = SchemaFactory
        .newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
    schema = schemaFactory.newSchema(schemaFile);
  }
    
  public ResultData createOrdinaryDataset1() {
    MockData data = new MockData();
    MockDataTextValue situation = new MockDataTextValue();
    MockDataTimeValue horizont = new MockDataTimeValue();
    MockDataTimeValue T = new MockDataTimeValue();
    MockData guete = new MockData();
    MockDataNumberValue index = new MockDataNumberValue();
    index.set(1);
    guete.setNumberValue("Index", index);
    situation.setText("Stau");
    horizont.setMillis(0);
    T.setMillis(0);
    data.setTextValue("Situation", situation);
    data.setTimeValue("Horizont", horizont);
    data.setTimeValue("T", T);
    data.addItem("Gte", guete);
    SystemObject obj = new MockSystemObject("pid.mock.ordinary1", "");
    return new ResultData(obj, null, false, 0L, 0L, (byte) 0, data);
  }

  // Unpassende Attributgruppe: Keine Werte abrufbar
  public ResultData createFaultDataset1() {
    ResultData d = createOrdinaryDataset1();
    d.getData().getTimeValue("T").setMillis(Long.MAX_VALUE);
    MockData data = new MockData();
    MockDataTextValue situation = new MockDataTextValue();
    MockDataTimeValue horizont = new MockDataTimeValue();
    MockDataTimeValue T = new MockDataTimeValue();
    MockData guete = new MockData();
    MockDataNumberValue index = new MockDataNumberValue();
    index.set(1);
    guete.setNumberValue("Index", index);
    situation.setText("Stau");
    horizont.setMillis(0);
    T.setMillis(0);
    data.setTextValue("Situation", situation);
    data.setTimeValue("Horizont", horizont);
    data.setTimeValue("T", T);
    data.addItem("Gte", guete);
    SystemObject obj = new MockSystemObject("pid.mock.fault1", "");
    return new ResultData(obj, null, false, 0L, 0L, (byte) 0, data);   
  }
  
  // Unpassende Attributgruppe: Keine Werte abrufbar
  public ResultData createFaultDataset2() {
    ResultData d = createOrdinaryDataset1();
    MockData data = (MockData) d.getData();
    SystemObject obj = new MockSystemObject("pid.mock.fault2", "");
    return new ResultData(obj, null, false, 0L, 0L, (byte) 1, data);   
  }

  public ResultData createFaultDataset3() {
    ResultData d = createOrdinaryDataset1();
    d.getData().getTimeValue("Horizont").setMillis(Long.MAX_VALUE);
    SystemObject obj = new MockSystemObject("pid.mock.fault3", "");
    return new ResultData(obj, null, false, 0L, 0L, (byte) 0, d.getData());   
  }


  public Object apply(MockPluginExport plugin,ObjekteAtgAsp objekteAtgAsp, Collection<ResultData> resultData) {
    Collection<ObjekteAtgAsp> objList = new ArrayList<>();
    objList.add(objekteAtgAsp);

    try {
      plugin.getDatexKonvertierungDynamisch().initialisiere(objList);
    } catch (ConfigurationException ex) {
      return null;
    }

    Map<ObjekteAtgAsp, Collection<ResultData> > data = new HashMap<>();

    data.put(objekteAtgAsp, resultData);
        
    return plugin.getDatexKonvertierungDynamisch().getDatexII(data);
  }
  
  public MockPluginExport setupPluginFull() {
    MockPluginExport mockPluginExport = new MockPluginExport();
    MockKonfigUndPersistenzModul konfig = new MockKonfigUndPersistenzModul();
    MockKonfigUndPersistenzModul persistenz = new MockKonfigUndPersistenzModul();
    mockPluginExport.setKonfigurationsModul(konfig);
    mockPluginExport.setPersistenzModul(persistenz);    
    
    mockPluginExport.setDatexKonvertierungDynamisch(new MQVerkehrslage(mockPluginExport));
    mockPluginExport.setDatexKonvertierungStatisch(new MQPredefinedLocation(mockPluginExport));
        
    konfig.putData("National-Identifier", "TestNI");
    konfig.putData("LocationGroupId", "TestGroupId");
    konfig.putData("StandardIntervall", "3.0");
    
    return mockPluginExport;
  }
  
  @Test
  public void test001() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

//  @Test
//  public void test002() throws Exception {
//    MockPluginExport plugin = setupPluginFull();
//    Object xml = apply(plugin, ordinaryObjSet, boundaryDataset);
//    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
//  }

  @Test
  public void test003() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, faultDataset1);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test004() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, faultDataset2);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

//  @Test
//  public void test005() throws Exception {
//    MockPluginExport plugin = setupPluginFull();
//    Object xml = apply(plugin, ordinaryObjSet, faultDataset3);
//    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
//  }
//
//  @Test
//  public void test006() throws Exception {
//    MockPluginExport plugin = setupPluginFull();
//    Object xml = apply(plugin, ordinaryObjSet, faultDataset4);
//    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
//  }
//
//  @Test
//  public void test007() throws Exception {
//    MockPluginExport plugin = setupPluginFull();
//    Object xml = apply(plugin, ordinaryObjSet, faultDataset5);
//    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
//  }

  @Test
  public void test008() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, faultObjSet1, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test009() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, faultObjSet2, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test010() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, emptyDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }
  

  @Test
  public void test011() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, faultDataset6);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }
  
  @Test
  public void test012() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).removeData("National-Identifier");
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertNull(xml);
  }
  
  @Test
  public void test013() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).removeData("LocationGroupId");
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertNull(xml);
  }
  
  @Test
  public void test014() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).removeData("StandardIntervall");
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }
  
  @Test
  public void test016() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).removeData("StandardIntervall");
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).putData("StandardIntervall", "QWERT");
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test017() throws Exception {
    for (Field f : WerkzeugSammlung.class.getDeclaredFields()) {
      if (f.getType().equals(DatatypeFactory.class)) {
        f.setAccessible(true);
        f.set(null,null);
        System.err.println("Reflection: Found and patched field.");
      }
    }
 
    System.getProperties().put("javax.xml.datatype.DatatypeFactory", "Object");
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    System.getProperties().remove("javax.xml.datatype.DatatypeFactory");
    Assert.assertNull(xml);
  }

//  @Test
//  public void test018() throws Exception {
//    MockPluginExport plugin = setupPluginFull();
//    Object xml = apply(plugin, ordinaryObjSet, faultDataset7);
//    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
//  }

  @Test
  public void test019() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    plugin.setDatexKonvertierungStatisch(null);
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test020() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    plugin.setDatexKonvertierungStatisch(null);
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test021() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Field f = plugin.getDatexKonvertierungStatisch().getClass().getDeclaredField("version");
    f.setAccessible(true);
    f.set(plugin.getDatexKonvertierungStatisch(), null);
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test022() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Field f = plugin.getDatexKonvertierungStatisch().getClass().getDeclaredField("version");
    f.setAccessible(true);
    f.set(plugin.getDatexKonvertierungStatisch(), "");
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test023() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    plugin.setDatexKonvertierungStatisch(new UDMeasurementSite(plugin));
    Object xml = apply(plugin, ordinaryObjSet, ordinaryDataset);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  public boolean validate(MockPluginExport plugin, Object xml, CanBeNull canBeNull) throws Exception {
    if (xml == null) {
      return canBeNull.equals(CanBeNull.CAN_BE_NULL); 
    }
    Marshaller marshaller = plugin.getDatexKonvertierungDynamisch().getJAXBContext().createMarshaller();
    marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, new Boolean(true));

    marshaller.marshal(new JAXBElement<Object>(new QName(DATEX2_NAMESPACE,DATEX2_ROOTTAG), Object.class, xml),System.out);

    ByteArrayOutputStream ba = new ByteArrayOutputStream();
    marshaller.marshal(new JAXBElement<Object>(new QName(DATEX2_NAMESPACE,DATEX2_ROOTTAG), Object.class, xml), ba);
    Source xmls = new StreamSource(new StringReader(ba.toString()));
    Validator validator = schema.newValidator();
    validator.validate(xmls);

    return true;
  }
}
